import { connect } from 'react-redux';
import { graphql, compose, withApollo } from 'react-apollo';
import { createSelector } from 'reselect';
import { setActiveJsonEditor, setNewLineAction } from '../../reducers';
import { changeEditAction } from '@files-stack/client-redux';
import { USER_PREFERENCES_QUERY, SAVE_USER_SETTINGS_MUTATE, SAVE_WORKSPACE_SETTINGS_MUTATE } from '../../graphql';
import {
    CustomSettings, ICustomSettingsApiProps, ICustomSettingsStateProps, 
    ICustomSettingsDispatchProps, ICustomSettings,
} from './CustomSettings';

const CustomSettingsWithData: React.ComponentClass<ICustomSettings> | React.StatelessComponent<{}> = compose(
    graphql<ICustomSettingsApiProps, any, {}, {}>(USER_PREFERENCES_QUERY, {
        props: createSelector(
            ({ ownProps, data }) => data,
            (data) => ({ userPref: data.userPreferences }),
        ),
    }),
    graphql<ICustomSettingsApiProps, any, {}, {}>(SAVE_USER_SETTINGS_MUTATE, { name: 'saveMutate' }),
    graphql<ICustomSettingsApiProps, any, {}, {}>(SAVE_WORKSPACE_SETTINGS_MUTATE, { name: 'updateConfigMutate' }),
)(withApollo(CustomSettings));

const mapStateToProps = (state: any): ICustomSettingsStateProps => ({
    activeId: state.preferences.active,
    newLine: state.preferences.newLine,
    rootPath: state['@files/rootpath'],
});

const mapDispatchToProps = (dispatch): ICustomSettingsDispatchProps => ({
    setActiveJsonEditor: (editorId: number) => dispatch(setActiveJsonEditor(editorId)),
    change: (changed: boolean) => dispatch(changeEditAction(changed)),
    setNewLine: (content: string) => dispatch(setNewLineAction(content)),
});

const CustomSettingsEnhanced: any = connect(mapStateToProps, mapDispatchToProps)(CustomSettingsWithData);

export default CustomSettingsEnhanced;
