import * as React from 'react';
import { connect as fela } from 'react-fela';
import * as enhanceWithClickOutside from 'react-click-outside';
import { map, isEqual } from 'lodash';

import { CHANGED_CONTENT_QUERY } from '@files-stack/graphql-gql';

import {
  getHints,
  insertNewContent,
  change1,
  change2,
  checkExist,
  change4,
} from '../../utils';

const COPY_TO_SETTINGS = 'Copy to settings';

export type IHelperPopupStateProps = {
  rootPath: string;
};

export type IHelperPopupDispatchProps = {
  change: (changed: boolean) => void;
  setNewLine: (content) => void;
};

export type IHelperPopupOwnProps = {
  styles: any;
  client: any;
  editorFile: any;
  editorContent: any;
  preference: any;
  activeId: number;
  onHideHints: any;
  open: boolean;
};

export type IHelperPopup = IHelperPopupStateProps &
  IHelperPopupDispatchProps &
  IHelperPopupOwnProps;

class HelperPopupView extends React.Component<IHelperPopup, {}> {
  // private handleClickOutside() {
  //     console.log('gere');
  //     const { open, onHideHints } = this.props;
  //     if (open) {
  //         onHideHints();
  //     }
  // }

  private mergeEvent(a, b) {
    const changes = [].concat(a.changes).concat(b.changes);
    const versionId = b.versionId;
    const isUndoing = a.isUndoing || b.isUndoing;
    const isRedoing = a.isRedoing || b.isRedoing;

    return { changes, versionId, isUndoing, isRedoing };
  }

  private onListItemSelect = value => {
    const {
      client,
      editorFile,
      editorContent,
      rootPath,
      preference,
      activeId,
      setNewLine,
      onHideHints,
    } = this.props;

    let insertedValue = value;
    if ((!preference.enum || !preference.enum.length) && preference.type === 'string') {
      insertedValue = preference.default;
    } else if (preference.type === 'object') {
        insertedValue = preference.default.replace(/\"/g, `'`);
    }
    const insertedStr = `    "${preference.name}": "${insertedValue}",`;

    if (activeId === 1) {
      let newValue, changedEvent, changedEventCurrent;
      const resource = `${rootPath}/.cdecode`;
      const checked = checkExist(editorContent.newValue, insertedStr);
      if (!checked) {
        newValue = insertNewContent(editorContent.newValue, insertedStr);

        const editorArr = editorContent.newValue.split('\n');
        const startLineNumber = editorArr.length;
        const startColumnNumber = editorArr[editorArr.length - 2].length;

        changedEventCurrent = this.mergeEvent(
          change1(startLineNumber, startColumnNumber),
          change2(insertedStr, startLineNumber),
        );
      } else {
        newValue = checked.value;
        const startLineNumber = checked.checkedIndex + 1;
        changedEventCurrent = change4(
          insertedStr,
          startLineNumber,
          checked.oldLength,
        );
      }
      if (!editorFile.changed) {
        this.props.change(true);
      }

      changedEvent = editorContent.changedEvent
        ? this.mergeEvent(editorContent.changedEvent, changedEventCurrent)
        : changedEventCurrent;
      try {
        const { changedContent } = client.readQuery({
          query: CHANGED_CONTENT_QUERY,
          variables: { resource },
        });

        changedContent.resource = resource;
        changedContent.changedEvent = changedEvent;
        changedContent.value = newValue;

        client.writeQuery({
          query: CHANGED_CONTENT_QUERY,
          variables: { resource },
          data: { changedContent },
        });
      } catch (exception) {
        // console.log(exception);
      }
    } else if (activeId === 0) {
      setNewLine(insertedStr);
    }
    onHideHints();
  }

  private renderPopupList = preference => {
    const { styles } = this.props;
    let hinstList = [COPY_TO_SETTINGS];
    if (preference.type !== 'object') {
      hinstList = getHints(preference);
    }

    return map(hinstList, (item, index) => {
      return (
        <div
          key={index}
          className={styles.listItem}
          onClick={() => this.onListItemSelect(item)}
        >
          {item}
        </div>
      );
    });
  }

  public render() {
    const { styles, open, preference } = this.props;
    if (!open) {
      return null;
    }

    return (
      <div className={styles.popup}>{this.renderPopupList(preference)}</div>
    );
  }
}

export const HelperPopup: any = fela<any, any>({
  popup: ({ hintPosition }) => ({
    padding: '10px 0',
    left: '10px',
    borderRadius: '8px',
    background: 'gray',
    zIndex: 1,
    top: hintPosition ? `${hintPosition - 80}px` : '10px',
    fontSize: '12px',
    position: 'absolute',
  }),
  listItem: () => ({
    cursor: 'pointer',
    padding: '2px 10px',
    ':hover': {
      background: 'blue',
    },
  }),
})(enhanceWithClickOutside(HelperPopupView));
