import * as React from 'react';
import { connect as fela, withTheme } from 'react-fela';

import { MonacoLoader } from '@workbench-stack/monaco-browser';

export type IUserCustomSettingsOwnProps = {
    styles: any;
    prefs: any;
    theme: any;
    activeId: number;
    onChange: any;
};

export type IUserCustomSettings = IUserCustomSettingsOwnProps;

class UserCustomSettingsView extends React.Component<IUserCustomSettings, {}> {
    private monaco: any;
    private editor: any;

    public componentWillReceiveProps(nextProps) {
        const { theme, activeEditorId, prefs } = nextProps;
        const { prefs: prevPrefs } = this.props;

        if (theme !== this.props.theme) {
            if (this.monaco && this.monaco.editor) {
                this.monaco.editor.setTheme(theme.className);
            }
        }
    }

    private editorDidMount = (editor, monaco) => {
        monaco.languages.registerCodeLensProvider('jsonc', {
            provideCodeLenses: function (model, token) {
                return [
                    {
                        range: {
                            startLineNumber: 1,
                            startColumn: 1,
                            endLineNumber: 2,
                            endColumn: 1,
                        },
                        id: 'TitleJsonc',
                        command: {
                            title: 'Place your settings here to overwrite the Default Settings',
                        },
                    },
                ];
            },
            resolveCodeLens: function (model, codeLens, token) {
                return codeLens;
            },
        });

        this.monaco = monaco;
        this.editor = editor;
    }

    public render() {
        const { styles, activeId, theme, prefs, onChange } = this.props;

        const editorOptions = {
            selectOnLineNumbers: false,
            readOnly: false,
            theme: 'vs-dark',
            automaticLayout: true,
        };

        if (theme) { editorOptions.theme = theme.className; }

        let value = '{\n}';
        if (prefs) {
            value = prefs.newValue && prefs.newValue !== '' ? prefs.newValue : prefs.content;
        }

        return (
            <div className={styles.container}>
                <MonacoLoader
                    width="100%"
                    height="100%"
                    language="jsonc"
                    value={value}
                    options={editorOptions}
                    onChange={onChange}
                    uri={'test/uri'}
                    enableLanguageService={true}
                    editorDidMount={this.editorDidMount}
                />
            </div>
        );
    }
}

export const UserCustomSettings: any = fela({
    container: () => ({
        height: '100%',
    }),
})(withTheme(UserCustomSettingsView));
