import * as React from 'react';
import { connect as fela, withTheme } from 'react-fela';

import { MonacoLoader } from '@workbench-stack/monaco-browser';

export type IWorkspaceCustomSettingsState = {
    content: any;
};

export type IWorkspaceCustomSettingsOwnProps = {
    styles: any;
    content: any;
    theme: any;
    activeId: number;
    onChange: any;
};

export type IWorkspaceCustomSettings =  IWorkspaceCustomSettingsOwnProps;

class WorkspaceCustomSettingsView extends React.Component<IWorkspaceCustomSettings, IWorkspaceCustomSettingsState> {
    private monaco: any;
    private editor: any;
    constructor(props) {
        super(props);

        this.state = {
            content: null,
        };
    }

    public componentWillMount() {
        const { content } = this.props;
        if (content) {
            this.setState({ content });
        }
    }

    public componentWillReceiveProps(nextProps) {
        const { theme, activeEditorId, content } = nextProps;
        const { content: prevContent } = this.props;
    
        if (theme !== this.props.theme) {
          if (this.monaco && this.monaco.editor) {
            this.monaco.editor.setTheme(theme.className);
          }
        }
    
        if (content.newValue !== prevContent.newValue) {
          this.setState({ content });
        }
    }

    private editorDidMount = (editor, monaco) => {
        // monaco.languages.registerCodeLensProvider('jsonc', {
        //     provideCodeLenses: function (model, token) {
        //         return [
        //             {
        //                 range: {
        //                     startLineNumber: 1,
        //                     startColumn: 1,
        //                     endLineNumber: 2,
        //                     endColumn: 1,
        //                 },
        //                 id: 'TitleJsonc',
        //                 command: {
        //                     title: 'Place your settings here to overwrite the Default Settings',
        //                 },
        //             },
        //         ];
        //     },
        //     resolveCodeLens: function (model, codeLens, token) {
        //         return codeLens;
        //     },
        // });

        this.monaco = monaco;
        this.editor = editor;
    }

    public render() {
        const { styles, activeId, theme, onChange } = this.props;
        const { content } = this.state;

        const editorOptions = {
            selectOnLineNumbers: false,
            readOnly: false,
            theme: 'vs-dark',
            automaticLayout: true,
        };

        if (theme) { editorOptions.theme = theme.className; }

        let value = content ? content.newValue : '';
        const uri = content.resource ? content.resource.replace('file://', '') : '';

        return (
            <div className={styles.container}>
                <MonacoLoader
                    width="100%"
                    height="100%"
                    language="jsonc"
                    value={value}
                    options={editorOptions}
                    onChange={onChange}
                    uri={uri}
                    enableLanguageService={true}
                    editorDidMount={this.editorDidMount}
                />
            </div>
        );
    }
}

export const WorkspaceCustomSettings: any = fela({
    container: () => ({
        height: '100%',
        marginTop: '10px',
    }),
})(withTheme(WorkspaceCustomSettingsView));
