import * as React from 'react';

import { connect as fela } from 'react-fela';
import * as Dimensions from 'react-dimensions';

import DefaultSettings from '../../components/DefaultSettings';
import CustomSettings from '../../components/CustomSettings';

export type ICustomJsonEditorApiProps = {
    defaultPrefs?: any;
};

export type ICustomJsonEditorStateProps = {
    defaultSettingsWidth: number;
};

export type ICustomJsonEditorDispatchProps = {
    resizeWidth: (width: number) => any;
};

export type ICustomJsonEditorOwnProps = {
    containerWidth: number;
    styles: any;
    content: any;
    client: any;
    editorFile: any;
    onKeyDown: any;
    onChange: any;
};

export type ICustomJsonEditorState = {
    search: string;
    defaultPrefs: any;
};

export type ICustomJsonEditor = ICustomJsonEditorApiProps & ICustomJsonEditorStateProps &
    ICustomJsonEditorDispatchProps & ICustomJsonEditorOwnProps;


class CustomJsonEditor extends React.Component<ICustomJsonEditor, ICustomJsonEditorState> {
    constructor(props) {
        super(props);

        this.state = {
            search: null,
            defaultPrefs: null,
        };
    }

    public componentWillMount() {
        if (this.props.defaultPrefs) {
            this.setState({ defaultPrefs: this.props.defaultPrefs.preferences });
        }
    }

    public componentWillReceiveProps(nextProps) {
        if (nextProps.defaultPrefs !== this.props.defaultPrefs) {
            this.setState({ defaultPrefs: nextProps.defaultPrefs.preferences });
        }
    }

    public componentDidMount() {
        this.props.resizeWidth(this.props.containerWidth / 2);
    }

    private onChange = (e) => {
        let defaultPrefs;
        if (e.target.value === '') {
            defaultPrefs = this.props.defaultPrefs.preferences;
        } else {
            defaultPrefs = this.props.defaultPrefs.preferences.filter(el => {
                return el.name.toLowerCase().indexOf(e.target.value) !== -1;
            });
        }
        this.setState({ search: e.target.value, defaultPrefs });
    }

    public render() {
        const { styles, onKeyDown, content, onChange, editorFile, client } = this.props;
        const { defaultPrefs } = this.state;
        return (
            <div className={styles.container}>
                <div className={styles.searchWrapper}>
                    <input placeholder="Search Settings" className={styles.search} onChange={this.onChange} />
                </div>
                <div className={styles.settings}>
                    <div className={styles.left}>
                        <DefaultSettings
                            editorContent={content}
                            editorFile={editorFile}
                            defaultPrefs={defaultPrefs}
                            client={client}
                        />
                    </div>
                    <div className={styles.right}>
                        <CustomSettings
                            onKeyDown={onKeyDown}
                            content={content}
                            onChange={onChange}
                        />
                    </div>
                </div>
            </div>
        );
    }
}

export default Dimensions()(fela<any, any>({
    container: () => ({
        width: '100%',
        height: '100%',
    }),
    settings: () => ({
        display: 'flex',
        height: '100%',
    }),
    left: ({ defaultSettingsWidth }) => ({
        height: '100%',
        width: `${defaultSettingsWidth}px`,
    }),
    right: ({ defaultSettingsWidth }) => ({
        height: '100%',
        width: `calc(100% - ${defaultSettingsWidth}px)`,
    }),
    search: () => ({
        width: '100%',
    }),
    searchWrapper: () => ({
        padding: '20px',
    }),
})(CustomJsonEditor));
