import { connect } from 'react-redux';
import { graphql, compose, withApollo } from 'react-apollo';
import { createSelector } from 'reselect';

import { DEFAULT_PREFERENCE_QUERY } from '../../graphql';
import { resizeWidthAction } from '../../reducers';

import CustomJsonEditor, {ICustomJsonEditorStateProps, 
    ICustomJsonEditorDispatchProps, ICustomJsonEditorApiProps, ICustomJsonEditor} from './CustomJsonEditor';

const CustomJsonEditorWithData: React.ComponentClass<ICustomJsonEditor> | React.StatelessComponent<{}> = compose(
    graphql<ICustomJsonEditorApiProps, any, {}, {}>(DEFAULT_PREFERENCE_QUERY, {
        props: createSelector(
            ({ ownProps, data }) => data,
            (data) => ({ defaultPrefs: data.defaultPreferences}),
        ),
    }),
)(withApollo(CustomJsonEditor));

const mapStateToProps = (state: any): ICustomJsonEditorStateProps => ({
    defaultSettingsWidth: state.preferences.width,
});
const mapDispatchToProps = (dispatch): ICustomJsonEditorDispatchProps => ({
    resizeWidth: (width: number) => dispatch(resizeWidthAction(width)),
});

const CustomJsonEditorEnhanced: any = connect(mapStateToProps, mapDispatchToProps)(CustomJsonEditorWithData);

export default CustomJsonEditorEnhanced;
