import { map, isPlainObject } from 'lodash';
import { parse } from '../../../common/jsonc-parser/main';

export const getContent = (settingsArray) => {
    let value = '';
    settingsArray.forEach(settings => {
        settings.data.forEach(el => {
            value += `//${el.description}\n`;
            if (el.type === 'object') {
                value += `"${el.name}": "${el.default.replace(/\"/g, `'`)}"\n\n`;
            } else {
                value += `"${el.name}": "${el.default}"\n\n`;
            } 
        });
    });

    return value;
};

export const findEditorLines = (domContent, settingsArr) => {
    const settings = this.transform(settingsArr);
    let settingsIndex = 0;
    let contentName = 'description';
    return map(domContent, (content, index) => {
        if (settingsIndex < settings.length) {
            const normaliseStr = settings[settingsIndex][contentName].replace(/\s+/g, ' ');
            const normaliseSubStr = content.replace(/\s+/g, ' ');
            const substrIndex = normaliseStr.indexOf(normaliseSubStr);
            if (substrIndex === -1 || contentName === 'empty') {
                settingsIndex++;
            }
            let res;
            if (contentName === 'empty') {
                res = { index: -1, content, display: false, i: index };
            } else {
                res = { index: settingsIndex, content, display: false, i: index };
            }

            if (substrIndex === -1 || substrIndex + normaliseSubStr.length === normaliseStr.length) {
                if (contentName === 'description') {
                    contentName = 'option';
                } else if (contentName === 'option') {
                    contentName = 'empty';
                } else if (contentName === 'empty') {
                    contentName = 'description';
                }
            }

            return res;
        } else {
            return { index: -1, content, display: false, i: index };
        }

    });
};


export const transform = (settingsArr) => {
    return map(settingsArr, item => {
        return { description: `//${item.description}`, option: `"${item.name}": "${item.default}"`, empty: '\n' };
    });
};

export const getHints = (settings) => {
    if (settings.type === 'boolean') {
        return ['true', 'false'];
    } else if (settings.type === 'string' && settings.enum && settings.enum.length) {
        return settings.enum;
    } else if (settings.type === 'string' && (!settings.enum || !settings.enum.length)) {
        return ['Copy to settings'];
    } else if (settings.type === 'number') {
        return ['Copy to settings'];
    }
};

export const insertNewContent = (content, newLine) => {
    const contentArray = content.split('\n');
    contentArray.pop();
    contentArray.push(newLine);
    contentArray.push('}');
    return contentArray.join('\n');
};

export const change1 = (startLineNumber, column) => {
    return {
        changes: [{
            range: {
                endColumn: column + 1,
                endLineNumber: startLineNumber - 1,
                startColumn: column + 1,
                startLineNumber: startLineNumber - 1,
            },
            rangeLength: 0,
            text: '\n    ',
        }],
        eol: '\n',
        isFlush: false,
        isRedoing: false,
        isUndoing: false,
        versionId: 2,
    };
};

export const change2 = (insertedStr, startLineNumber) => {
    return {
        changes: [{
            range: {
                endColumn: 1,
                endLineNumber: startLineNumber,
                startColumn: 1,
                startLineNumber: startLineNumber,
            },
            rangeLength: 0,
            text: insertedStr,
        }],
        eol: '\n',
        isFlush: false,
        isRedoing: false,
        isUndoing: false,
        versionId: 3,
    };
};

export const change4 = (insertedStr, startLineNumber, oldLength) => {
    return {
        changes: [{
            range: {
                endColumn: oldLength + 1,
                endLineNumber: startLineNumber,
                startColumn: 1,
                startLineNumber: startLineNumber,
            },
            rangeLength: oldLength - 4,
            text: insertedStr,
        }],
        eol: '\n',
        isFlush: false,
        isRedoing: false,
        isUndoing: false,
        versionId: 3,
    };
};

export const countSettingsLength = (preferences) => {
    let length = 0;
    if (!preferences) {
        return length;
    }
    preferences.forEach(el => {
        length += el.data.length;
    });
    return length;
};

export const getDeltaDecorations = (monaco, preferences) => {
    const preferencesLength = countSettingsLength(preferences);
    const deltaDecoration = [];
    for (let i = 0; i < preferencesLength * 3; i = i + 3) {
        deltaDecoration.push({
            range: new monaco.Range(i + 2, 1, i + 2, 1),
            options: {
                glyphMarginClassName: 'line-hint',
            },
        });
    }
    return deltaDecoration;
};

export const checkExist = (content, newLine) => {
    const key = parse(newLine);
    const jsonContent = parse(content);
    const value = jsonContent[key];
    let checkedIndex = 0;
    let oldLength = 0;

    if (value !== null && value !== undefined) {
        const lines = content.split('\n');
        const newArr = map(lines, (el, index) => {
            if (el.indexOf(value) !== -1) {
                checkedIndex = index;
                oldLength = el.length;
                return newLine;
            }
            return el;
        });
        return { value: newArr.join('\n'), checkedIndex, oldLength };
    }
    return false;
};

export const addCategories = (prefs, editor, viewZones) => {
    let position = 0;
    const viewZoneNew = [];
    prefs.forEach(key => {
        let domNode = document.createElement('div');
        domNode.className = 'widget-title';
        domNode.innerHTML = key.type.charAt(0).toUpperCase() + key.type.substr(1);

        const some = editor.changeViewZones(zones => {
            if (viewZones) {
                viewZones.forEach(el => zones.removeZone(el));
            }
            const res = zones.addZone({ afterLineNumber: position, domNode });
            viewZoneNew.push(res);
        });
        position += 3 * key.data.length;
    });
    return viewZoneNew;
};


export const getPrefsList = (prefs) => {
    let list = [];

    prefs.forEach(el => {
        list = [...list, ...el.data];
    });
    return list;
};
